
local Recount2 = Recount2
local SM = LibStub:GetLibrary("LibSharedMedia-3.0")

local tinsert = table.insert
local tconcat = table.concat
local string_match = string.match
local bit_band = bit.band

local function SetFontSize(string, size)
	local Font, Height, Flags = string:GetFont()
	string:SetFont(Font, size, Flags)
end


local detailTable

local function namePass(name)
	Recount2:AddExpressionConstant('cName', name)
	return name
end

local function nameGUID(guid)
	return Recount2:SetGUIDInfo(guid)
end

local function nameAbility(ability)
	local name
	if string.match(ability, "(%d+)") then
		name, rank = GetSpellInfo(tonumber(ability))
		if rank~="" then
			name = name.." ("..rank..")"
		end
	else
		name = ability
	end
	Recount2:AddExpressionConstant('cName', name)
	return name
end

local function FixCaps(capsstr)
	if type(capsstr)=="string" then
		return string.upper(string.sub(capsstr,1,1))..string.lower(string.sub(capsstr,2))
	else
		return nil
	end
end

local function nameClass(class)
	local name = FixCaps(class)
	Recount2:AddExpressionConstant('cName', name)
	return name
end

local function nameHitType(hitType)
	local name
	local nameTable = Recount2:GetTable()
	for k, v in pairs(Recount2.GroupNames.Type) do
		if bit_band(k, hitType)==k then
			tinsert(nameTable, v)
		end
	end

	if #nameTable>0 then
		name = tconcat(nameTable, " and ")
	else
		name = "Unknown Hit Type"
	end
	Recount2:FreeTable(nameTable)
	Recount2:AddExpressionConstant('cName', name)
	return name
end

local function nameSchool(school)
	local nameTable = Recount2:GetTable()
	for k, v in pairs(Recount2.GroupNames.School) do
		if bit_band(k, school)==k then
			tinsert(nameTable, v)
		end
	end

	if #nameTable>0 then
		name = tconcat(nameTable, " and ")
	else
		name = "Unknown School"
	end
	Recount2:FreeTable(nameTable)
	Recount2:AddExpressionConstant('cName', name)
	return name
end

local function nameSex(sex)
	if sex==1 then
		name = "Neuter"
	elseif sex==2 then
		name = "Male"
	elseif sex==3 then
		name = "Female"
	else
		name = sex
	end
	Recount2:AddExpressionConstant('cName', name)
	return name
end

local nameFuncs={
	["Ability"] =	nameAbility,
	["Class"] =	nameClass,
	["Info"] =	namePass,
	["Hit Type"] =	nameHitType,
	["School"] =	nameSchool,
	["Source GUID"] =	nameGUID,
	["Source Name"] =	namePass,
	["Source Level"] =	namePass,
	["Source Race"] =	namePass,
	["Source Class"] =	nameClass,
	["Source Classification"] =	namePass,
	["Source Creature Type"] =	namePass,
	["Source Creature Family"] =	namePass,
	["Source Faction"] =	namePass,
	["Source Sex"] =	nameSex,
	["Source Trivial"] =	namePass,
	["Target GUID"] =	nameGUID,
	["Target Name"] =	namePass,
	["Target Level"] =	namePass,
	["Target Race"] =	namePass,
	["Target Class"] =	nameClass,
	["Target Classification"] =	namePass,
	["Target Creature Type"] =	namePass,
	["Target Creature Family"] =	namePass,
	["Target Faction"] =	namePass,
	["Target Sex"] =	nameSex,
	["Target Trivial"] =	namePass,
}

local function ClickedRow(self)
	if self.clickFunc then
		self.clickFunc(self.who, self.dataTable, unpack(self.clickArgs))
	end
end

local function CreateRow(parent, rowWidth, rowHeight, clickFunc, clickArgs)
	--Create Frame
	local row=CreateFrame("Button", nil, parent)
	row:SetWidth(rowWidth)
	row:SetHeight(rowHeight)
	if clickFunc then
		row.ClickedRow = ClickedRow
		row:SetScript("OnMouseDown", row.ClickedRow)
		row.clickFunc = clickFunc
		row.clickArgs = clickArgs
		row:EnableMouse()
	end

	--Create Bar
	row.StatusBar = CreateFrame("StatusBar",nil,row)
	row.StatusBar:SetAllPoints(row)

	local BarTexture=SM:Fetch("statusbar","BantoBar")

	row.StatusBar:SetStatusBarTexture(BarTexture)
	row.StatusBar:SetStatusBarColor(.5, .5, .5, 1)
	row.StatusBar:SetMinMaxValues(0,100)
	row.StatusBar:SetValue(100)
	row.StatusBar:Show()
	
	

	--Create Right Text
	row.RightText=row.StatusBar:CreateFontString(nil,"OVERLAY","GameFontHighlightSmall")
	row.RightText:SetPoint("RIGHT", row.StatusBar,"RIGHT",-2, 0)
	row.RightText:SetJustifyH("RIGHT")
	row.RightText:SetText("0")
	row.RightText:SetTextColor(1,1,1,1)
	Recount2:AddFontString(row.RightText)
	SetFontSize(row.RightText,rowHeight*0.8)

	--Create Left Text
	row.LeftText=row.StatusBar:CreateFontString(nil,"OVERLAY","GameFontHighlightSmall")
	row.LeftText:SetPoint("LEFT", row.StatusBar,"LEFT", 2, 0)
	row.LeftText:SetPoint("RIGHT", row.RightText,"LEFT", -2, 0)
	row.LeftText:SetJustifyH("LEFT")
	row.LeftText:SetText("Test")
	row.LeftText:SetTextColor(1,1,1,1)
	Recount2:AddFontString(row.LeftText)
	SetFontSize(row.LeftText,rowHeight*0.8)

	return row
end

local function UpdateRowCount(self)
	local numRows = math.floor((self.lastHeight+self.rowSpacing)/(self.rowHeight+self.rowSpacing))

	local self_rows = self.rows

	self.numRows = numRows
	if numRows<#self_rows then
		for rowID = numRows+1, #self_rows do
			self_rows[rowID]:Hide()
		end
	elseif numRows>#self_rows then
		local w = self.lastWidth
		local rowHeight = self.rowHeight
		local rowSpace = self.rowHeight + self.rowSpacing
		for rowID = #self_rows+1, numRows do
			tinsert(self_rows, CreateRow(self, w, rowHeight, self.clickFunc, self.clickArgs))
			self_rows[#self_rows]:SetPoint("TOPLEFT", self, "TOPLEFT", 0, -(rowID-1)*rowSpace)
			
			if not self.vertScroll:IsShown() then
				self_rows[#self_rows]:SetPoint("TOPRIGHT", self, "TOPRIGHT", 0, -(rowID-1)*rowSpace)
			else
				self_rows[#self_rows]:SetPoint("TOPRIGHT", self, "TOPRIGHT", -rowSpace, -(rowID-1)*rowSpace)
			end
		end
	end
end

local function OnSizeChanged(self)
	local w = self:GetRight()-self:GetLeft()
	local h = self:GetTop()-self:GetBottom()


	if w~=self.lastWidth then
		for _, row in ipairs(self.rows) do
			row:SetWidth(w)
		end

		self.lastWidth = w
	end

	if h~=self.lastHeight then
		self.lastHeight = h
		self:UpdateRowCount()
		self:DisplayRows()
		self:CheckScrollbar()
	end
end





local function BarSort(a, b)
	if a.value==b.value then
		return tostring(a.key)<tostring(b.key)
	end
	return a.value>b.value
end

local SetCombatantConstants = Recount2.SetCombatantConstants
local function OnUpdateRows(self)
	if self.view then
		local view = self.view
		local filterData = Recount2.viewCompress[self.name]
		local cSort = self.cSort
		local cData = self.cData
		local needsUpdate = false or self.forceUpdate

		self.forceUpdate = nil


		
		--Need to calculate all the values we will be sorting by now
		local barValue = view.barValue
		local totalAmount = self.totalAmount
		local groupUpdates = Recount2.filter[self.name].groupUpdates

		for key in pairs(groupUpdates) do
			local vCombatant = filterData[key]
			groupUpdates[key] = nil

			SetCombatantConstants(vCombatant)
			local value = barValue()

			if value and value>0 then
				local combatant = cData[key]

				if combatant == nil then
					cData[key] = Recount2:GetTable()
					combatant = cData[key]
					combatant.key = key
					combatant.data = vCombatant
					combatant.value = 0
					combatant.uniqueLeftID = "left"..key..self.name
					combatant.uniqueRightID = "right"..key..self.name
					--Need to insert into cSort since it doesn't exist yet
					tinsert(cSort, combatant)
				end

				if value~=combatant.value then
					needsUpdate = true

					totalAmount = totalAmount - combatant.value
					combatant.value = value
					totalAmount = totalAmount + combatant.value
				end				
			end
		end

		if needsUpdate then
			--Sort!
			table.sort(cSort, BarSort)
	
			local MaxValue	
			if cSort[1] then
				MaxValue = cSort[1].value
			end
			
			self.totalAmount = totalAmount
			self.maxValue = MaxValue

			self:DisplayRows()
			self:CheckScrollbar()
		end
	else
		local self_rows = self.rows
		for i = 1, self.numRows do
			self_rows[i]:Hide()
		end
	end
end

local function CheckScrollbar(self)
	if #self.cSort>self.numRows then
		self.vertScroll:SetMinMaxValues(0, #self.cSort-self.numRows)
		if not self.vertScroll:IsShown() then
			self.vertScroll:Show()
			local self_rows = self.rows
			local newX = - self.rowHeight - self.rowSpacing
			local rowSpace = self.rowHeight + self.rowSpacing
			for i, row in ipairs(self_rows) do
				row:SetPoint("TOPRIGHT", self, "TOPRIGHT", newX, -(i-1)*rowSpace)
			end
		end
	elseif self.vertScroll:IsShown() then
		self.vertScroll:Hide()
		local rowSpace = self.rowHeight + self.rowSpacing
		local self_rows = self.rows
		for i, row in ipairs(self_rows) do
			row:SetPoint("TOPRIGHT", self, "TOPRIGHT", 0, -(i-1)*rowSpace)
		end
	end
end

local function DisplayRows(self)
	if self.view then
		local view = self.view
		local offset = 0
		local self_rows = self.rows
		local cSort = self.cSort
		local MaxValue = self.maxValue or 0
		local filterTable = Recount2.viewTable[self.name]

		if self.vertScroll:IsShown() then
			offset = math.floor(self.vertScroll:GetValue()+0.5)
		end

		Recount2:AddExpressionConstants("lTotal", self.totalAmount, "lCount", #cSort, "lMaxValue", MaxValue)
		MaxValue = MaxValue/100
		for i = 1, self.numRows do
			local row = self_rows[i]
			local combatant = cSort[i+offset]
			if combatant then
				SetCombatantConstants(combatant.data)

				--Set the name data for this row
				row.who = nameFuncs[view.groupOn](combatant.key)

				Recount2:AddExpressionConstant("cPos", i+offset)
				row.LeftText:SetText(Recount2:ProcessCompiledTags(view.leftText, combatant.uniqueLeftID))
				row.RightText:SetText(Recount2:ProcessCompiledTags(view.rightText, combatant.uniqueRightID))
				row.StatusBar:SetValue(combatant.value/MaxValue)
				row.dataTable = filterTable[combatant.key]
				row:Show()
			else
				row.dataTable = nil
				row:Hide()
			end
		end
	end
end

local function Report(self, title, num, where)
	Recount2:AddExpressionConstants("lTotal", self.totalAmount, "lCount", #self.cSort, "lMaxValue", self.maxValue or 0)
	Recount2:Report(title, self.cSort, num, nameFuncs[self.view.groupOn], self.view.leftText, self.view.rightText, where)
end

local function OnMouseWheel(self, arg1)
	if arg1==1 then
		self.vertScroll:Decrease()
	elseif arg1==-1 then
		self.vertScroll:Increase()
	end
end

local function SetView(self, barValue, leftText, rightText, dataSource, srcCombatant, dstCombatant, infoFilter, powerFilter, schoolFilter, typeFilter, groupOn)
	Recount2:SetFilter(self.name, dataSource, srcCombatant, dstCombatant, infoFilter, powerFilter, schoolFilter, typeFilter, groupOn, self.ScheduleUpdate, self)
	
	if not self.view then
		self.view = Recount2:GetTable()
	end
	
	local view = self.view

	view.barValue = Recount2:CompileExpression(barValue)

	if view.leftText then
		Recount2:FreeTableRecurse(view.leftText)
	end
	if view.rightText then
		Recount2:FreeTableRecurse(view.rightText)
	end
	
	view.leftText = Recount2:CompileTags(leftText)
	view.rightText = Recount2:CompileTags(rightText)

	if groupOn == nil then
		groupOn = "Source GUID"
	end
	view.groupOn = groupOn

	--Need to force a refresh
	self:Refresh()
end

local function Refresh(self)
	local cSort = self.cSort
	local cData = self.cData

	for k in ipairs(cSort) do
		cSort[k] = nil
	end

	for k, v in pairs(cData) do
		Recount2:FreeTable(v)
		cData[k] = nil
	end

	for _, v in ipairs(self.rows) do
		v.dataTable = nil
	end

	self.totalAmount = 0
	self.vertScroll:SetValue(0)
	self.forceUpdate = true
	self:OnUpdateRows()
end

local function ListOnHide(self)
	Recount2:RemoveTimer(self.updateName)
	Recount2:ClearFilter(self.name)
	self.view = nil

	local cSort = self.cSort
	local cData = self.cData

	for k in ipairs(cSort) do
		
		cSort[k] = nil
	end

	for k, v in pairs(cData) do
		Recount2:FreeTable(v)
		cData[k] = nil
	end

	for _, v in ipairs(self.rows) do
		v.dataTable = nil
	end
end

local function ListOnShow(self)
	self:OnUpdateRows()
	self:ScheduleUpdate()
end

local GetTime = GetTime
local function ScheduleUpdate(self)
	if self.nextScheduledUpdate<GetTime() then
		Recount2:ScheduleTimer(self.updateName, self.OnUpdateRows, 1, self)
	end
end

function Recount2:CreateListFrame(name, parent, width, height, rowHeight, rowSpacing, clickFunc, clickArgs)
	--Just store here so we can track for Expo
	SetCombatantConstants = Recount2.SetCombatantConstants

	local listFrame=CreateFrame("Frame", nil, parent)
	listFrame:SetWidth(width)
	listFrame:SetHeight(height)
	listFrame:SetPoint("CENTER")

	listFrame.name = name
	listFrame.lastWidth = width
	listFrame.lastHeight = height

	listFrame.rowHeight = rowHeight
	listFrame.rowSpacing = rowSpacing

	listFrame.clickFunc = clickFunc
	listFrame.clickArgs = clickArgs

	local numRows = math.floor((height+rowSpacing)/(rowHeight+rowSpacing))

	listFrame.numRows = numRows

	listFrame.rows = Recount2:GetTable()
	local listFrame_rows = listFrame.rows

	local rowSpace = rowSpacing+rowHeight
	for i = 1, numRows do
		tinsert(listFrame_rows, CreateRow(listFrame, width, rowHeight, clickFunc, clickArgs))
		listFrame_rows[i]:SetPoint("TOPLEFT", listFrame, "TOPLEFT", 0, -(i-1)*rowSpace)
		listFrame_rows[i]:SetPoint("TOPRIGHT", listFrame, "TOPRIGHT", 0, -(i-1)*rowSpace)
	end

	

	listFrame.cSort = Recount2:GetTable()
	listFrame.cData = Recount2:GetTable()

	listFrame.OnUpdateRows = OnUpdateRows
	listFrame.OnSizeChanged = OnSizeChanged
	listFrame.UpdateRowCount = UpdateRowCount
	listFrame.SetView = SetView
	listFrame.DisplayRows = DisplayRows
	listFrame.CheckScrollbar = CheckScrollbar
	listFrame.Refresh = Refresh
	listFrame.Report = Report

	listFrame.vertScroll = Recount2.Widgets:CreateScrollbar(listFrame, rowHeight, height, "VERTICAL")
	listFrame.vertScroll:SetPoint("TOPRIGHT", listFrame, "TOPRIGHT")
	listFrame.vertScroll:SetPoint("BOTTOMRIGHT", listFrame, "BOTTOMRIGHT")
	listFrame.vertScroll:SetOnChange(listFrame.DisplayRows, listFrame)
	listFrame.vertScroll:SetValue(0)
	listFrame.vertScroll:Hide()

	listFrame.OnMouseWheel = OnMouseWheel
	listFrame:EnableMouseWheel()
	listFrame:SetScript("OnMouseWheel", listFrame.OnMouseWheel)

	listFrame:SetScript("OnSizeChanged", listFrame.OnSizeChanged)

	listFrame.OnHide = ListOnHide
	listFrame.OnShow = ListOnShow
	listFrame:SetScript("OnHide", listFrame.OnHide)
	listFrame:SetScript("OnShow", listFrame.OnShow)

	listFrame:OnUpdateRows()

	listFrame.updateName = "LISTFRAME "..name
	listFrame.ScheduleUpdate = ScheduleUpdate
	listFrame.nextScheduledUpdate = GetTime()

	return listFrame
end
