--TODO: Store Unit's targets to be reused during target scans
local Recount2 = Recount2
local TARGET_SCAN_TIMER	= 3

local string_sub = string.sub
local string_format = string.format
local bit_band = bit.band

local GetNumPartyMembers = GetNumPartyMembers
local GetNumRaidMembers  = GetNumRaidMembers

local UnitGUID = UnitGUID
local UnitClass	= UnitClassBase
local UnitLevel	= UnitLevel
local UnitRace	= UnitRace
local UnitClassification= UnitClassification
local UnitCreatureType	= UnitCreatureType
local UnitCreatureFamily= UnitCreatureFamily
local UnitFactionGroup	= UnitFactionGroup
local UnitSex	= UnitSex
local UnitIsTrivial = UnitIsTrivial

local NPCData
local combatants

function Recount2:InitGUIDTracking()
	NPCData = self.Data.NPCData
	combatants = self.Data.Combatants
end

function Recount2:RegisterGUIDTracking()
	self:RegisterEvent("UPDATE_MOUSEOVER_UNIT", Recount2.MouseoverUnitChanged)
	self:RegisterEvent("UNIT_TARGET", Recount2.UnitTargetChanged)
end

function Recount2:UnregisterGUIDTracking()
	self:UnregisterEvent("UPDATE_MOUSEOVER_UNIT")
	self:UnregisterEvent("UNIT_TARGET")
end

function Recount2:GetAttributeTable(guid)
	--First check if this is an NPC or not
	local unitType = string_sub(guid, 3, 6)
	local attrTable, npcTable

	if unitType=="F130" then
		--NPC
		local npcID = tonumber(string_sub(guid, 7, 12), 16)
		attrTable = NPCData[npcID]

		if attrTable == nil then
			attrTable = self:GetTable()
			NPCData[npcID] = attrTable
			attrTable.notInit = true
		end
		npcTable = true
	else
		attrTable = combatants[guid].attrTable

		if attrTable == nil then
			attrTable = self:GetTable()
			combatants[guid].attrTable = attrTable
			attrTable.notInit = true
		end
	end
	return attrTable, npcTable
end

local scanCache = {}
local nextScan = 0

local function FindUnit(guid, name, flags, timestamp)
	--Is this our target?
	if bit_band(flags, 0x10000) == 0x10000 then 
		return "TARGET"
	--Is this our focus?
	elseif bit_band(flags, 0x20000) == 0x20000 then
		return "FOCUS"
	end
	
	--Check the affiliation
	local affil = bit_band(flags, 0xf)
	if affil~=0x8 then
		local pet    = bit_band(flags, 0x1000) == 0x1000 and "PET" or ""
		if UnitGUID(name)==guid then --We might be able to use name as the unit for the moment
			return name
		elseif affil==0x1 then --Am I the owner?
			if UnitGUID((pet=="PET" and pet) or "PLAYER")==guid then
				return (pet=="PET" and pet) or "PLAYER"
			end
		elseif affil==0x2 then --Is it in the party?
			local checking = "PARTY"..pet.."%d"
			for i = 1, GetNumPartyMembers() do
				if UnitGUID(string_format(checking, i))==guid then
					return string_format(checking, i)
				end
			end
		elseif affil==0x4 then --Is it in the raid?		
			local checking = "RAID"..pet.."%d"
			for i = 1, GetNumRaidMembers() do
				if UnitGUID(string_format(checking, i))==guid then
					return string_format(checking, i)
				end
			end	
		--Is this none?
		elseif flags == -2147483648 then
			return "none"
		end
	end

	--Last Resort is to target scan though don't want to do this too often
	if (combatants[guid].nextScan==nil) or (combatants[guid].nextScan<timestamp) then
		--Update the scan cache if its been long enough
		if nextScan<timestamp then
			for i = 1, GetNumPartyMembers() do
				local unit = string_format("PARTY%dTARGET", i)
				scanCache[unit] = UnitGUID(unit)
				scanCache[unit.."TARGET"] = UnitGUID(unit.."TARGET") --Maybe one of our targets is targeting it? mainly for things like guardians
				unit = string_format("PARTYPET%dTARGET", i)
				scanCache[unit] = UnitGUID(unit)
			end
			for i = 1, GetNumRaidMembers() do
				local unit = string_format("RAID%dTARGET", i)
				scanCache[unit] = UnitGUID(unit)
				scanCache[unit.."TARGET"] = UnitGUID(unit.."TARGET") --Maybe one of our targets is targeting it? mainly for things like guardians
				unit = string_format("RAIDPET%dTARGET", i)
				scanCache[unit] = UnitGUID(unit)
			end
			nextScan = timestamp + TARGET_SCAN_TIMER
		end

		--Scan the cache
		for unit, unitGuid in pairs(scanCache) do
			if guid==unitGuid then
				--Make sure this is still valid
				local unitGuid = UnitGUID(unit)
				if guid==unitGuid then
					return unit
				end
				scanCache[unit] = unitGuid
			end
		end

		--Can scan next time the cache needs to be updated
		combatants[guid].nextScan = nextScan
	end

end

local function UpdateAttrTable(attrTable, unit)
	attrTable.notInit = nil
	_, attrTable.class = UnitClass(unit)
	attrTable.level = UnitLevel(unit)
	_, attrTable.race = UnitRace(unit)
	attrTable.classification = UnitClassification(unit)
	attrTable.creatureType = UnitCreatureType(unit)
	attrTable.creatureFamily = UnitCreatureFamily(unit)
	attrTable.faction = UnitFactionGroup(unit)
	attrTable.sex = UnitSex(unit)
	attrTable.trivial = UnitIsTrivial(unit)
	attrTable.lastUnit = unit
end

local affiliationLookup = {
	[0x1] = 'Mine',
	[0x2] = 'Party',
	[0x4] = 'Raid',
	[0x8] = 'Others',
}

local reactionLookup = {
	[0x10] = 'Friendly',
	[0x20] = 'Neutral',
	[0x40] = 'Hostile',
}

local function UpdateRelationship(attrTable, flags)
	local affil = bit_band(flags, 0xf)
	local reaction = bit_band(flags, 0xf0)

	attrTable.affiliation = affiliationLookup[affil]
	attrTable.reaction = reactionLookup[reaction]
	attrTable.prevFlags = flags
end

function Recount2:FindUnitFromGUID(guid, name, flags, timestamp)
	local attrTable = combatants[guid].attrTable
	if attrTable ~= nil and not attrTable.notInit then
		--Make sure this is still the correct unit
		if UnitGUID(attrTable.lastUnit)==guid then
			return attrTable.lastUnit
		else
			unit = FindUnit(guid, name, flags, timestamp)
			if unit then
				attrTable.lastUnit = unit
			end
			return unit
		end
	end
	return FindUnit(guid, name, flags, timestamp)
end

local newAttrTable={}
function Recount2:UpdateGUIDInfo(guid, name, flags, timestamp, special)	
	if combatants[guid]==nil then
		--Initialize 
		combatants[guid] = self:GetTable()
	end

	local attrTable = combatants[guid].attrTable

	if attrTable == nil then
		attrTable, combatants[guid].npcTable = self:GetAttributeTable(guid)
		combatants[guid].attrTable = attrTable
		combatants[guid].attrTable.name = name
	end

	--Update the relationships if the flags are different
	if flags~=attrTable.prevFlags then
		UpdateRelationship(attrTable, flags)
	end

	--Need to switch over to our own mechanics for handling this 
	--Don't process anything that is "special"
	if not special and attrTable.notInit then
		local unit = FindUnit(guid, name, flags, timestamp)

		if unit then
			UpdateAttrTable(attrTable, unit)

			--if we init the table then this isn't a npc table as far as we are concerned but ours
			combatants[guid].npcTable = nil

			--No longer will be scanning for unit since have our attributes!
			combatants[guid].nextScan = nil
		end
	--If this is a NPC table then we should try to find our exact attributes
	elseif combatants[guid].npcTable then
		local unit = FindUnit(guid, name, flags, timestamp)

		if unit then
			newAttrTable.name = name
			UpdateAttrTable(newAttrTable, unit)

			--Compare the attribute tables if they are equal then we don't really want to use this new table
			local areEqual=true
			for k, v in pairs(attrTable) do
				if v~=newAttrTable[k] then
					areEqual = false
				end
			end

			for k, v in pairs(newAttrTable) do
				if v~=attrTable[k] then
					areEqual = false
				end
			end
			
			if not areEqual then
				combatants[guid].attrTable = newAttrTable
				newAttrTable = self:GetTable()
			end
			
			--We want to consider this now the correct attributes no matter what
			combatants[guid].npcTable = nil

			--No longer will be scanning for unit since have our attributes!
			combatants[guid].nextScan = nil
		end
	end
end


function Recount2:MouseoverUnitChanged()
	local unit = "mouseover"
	local guid = UnitGUID(unit)

	if combatants[guid]~=nil then
		local attrTable = combatants[guid].attrTable

		if attrTable.notInit then
			UpdateAttrTable(attrTable, unit)

			--if we init the table then this isn't a npc table as far as we are concerned but ours
			combatants[guid].npcTable = nil

			--No longer will be scanning for unit since have our attributes!
			combatants[guid].nextScan = nil
		elseif combatants[guid].npcTable then
			newAttrTable.name = UnitName(unit)
			UpdateAttrTable(newAttrTable, unit)

			--Compare the attribute tables if they are equal then we don't really want to use this new table
			local areEqual=true
			for k, v in pairs(attrTable) do
				if v~=newAttrTable[k] then
					areEqual = false
				end
			end

			for k, v in pairs(newAttrTable) do
				if v~=attrTable[k] then
					areEqual = false
				end
			end
			
			if not areEqual then
				combatants[guid].attrTable = newAttrTable
				newAttrTable = self:GetTable()
			end
			
			--We want to consider this now the correct attributes no matter what
			combatants[guid].npcTable = nil

			--No longer will be scanning for unit since have our attributes!
			combatants[guid].nextScan = nil
		end
	end
end

function Recount2:UnitTargetChanged(event, whosTarget)
	local unit = whosTarget.."target"
	local guid = UnitGUID(unit)

	if combatants[guid]~=nil then
		local attrTable = combatants[guid].attrTable

		if attrTable.notInit then
			UpdateAttrTable(attrTable, unit)

			--if we init the table then this isn't a npc table as far as we are concerned but ours
			combatants[guid].npcTable = nil

			--No longer will be scanning for unit since have our attributes!
			combatants[guid].nextScan = nil
		elseif combatants[guid].npcTable then
			newAttrTable.name = UnitName(unit)
			UpdateAttrTable(newAttrTable, unit)

			--Compare the attribute tables if they are equal then we don't really want to use this new table
			local areEqual=true
			for k, v in pairs(attrTable) do
				if v~=newAttrTable[k] then
					areEqual = false
				end
			end

			for k, v in pairs(newAttrTable) do
				if v~=attrTable[k] then
					areEqual = false
				end
			end
			
			if not areEqual then
				combatants[guid].attrTable = newAttrTable
				newAttrTable = self:GetTable()
			end
			
			--We want to consider this now the correct attributes no matter what
			combatants[guid].npcTable = nil

			--No longer will be scanning for unit since have our attributes!
			combatants[guid].nextScan = nil
		end
	end
end