local Recount2_Widgets = Recount2.Widgets

local CHECKED_TEXTURE = "Interface\\Addons\\Recount2\\Textures\\radiobutton-true.tga"
local UNCHECKED_TEXTURE = "Interface\\Addons\\Recount2\\Textures\\radiobutton-false.tga"

local bit_band = bit.band
local bit_bor = bit.bor
local bit_bnot = bit.bnot

local radioValues = {}
local radioFrames = {}
local radioFramesByValue = {}

local function SetFontSize(string, size)
	local Font, Height, Flags = string:GetFont()
	string:SetFont(Font, size, Flags)
end

local function GetValue(self)
	return self.value
end

local function SetValue(self, value)
	if value then
		if self.radioID then
			radioFrames[self.radioID]:SetValue(false)
			radioFrames[self.radioID] = self
			radioValues[self.radioID] = self.radioValue
		end

		self.checkImage:SetTexture(CHECKED_TEXTURE)
	else
		self.checkImage:SetTexture(UNCHECKED_TEXTURE)
	end

	--Set Last due to potentially be overwritten when you set the value
	self.value = value
end

local function OnEnter(self)
	self.background:Show()
end

local function OnLeave(self)
	self.background:Hide()
end

local function OnMouseDown(self)
	self:SetValue(true)
end

function Recount2_Widgets:CreateRadioButton(parent, width, height, text, radioID, radioValue)
	local radiobutton = CreateFrame("FRAME", nil, parent)

	radiobutton:SetWidth(width)
	radiobutton:SetHeight(height)

	radiobutton.background = radiobutton:CreateTexture(nil, "BACKGROUND")
	radiobutton.background:SetAllPoints(radiobutton)
	radiobutton.background:SetTexture(1.0, 1.0, 0.0, 0.25)
	radiobutton.background:Hide()

	radiobutton.checkImage = radiobutton:CreateTexture(nil, "OVERLAY")
	radiobutton.checkImage:SetPoint("TOPLEFT", radiobutton, "TOPLEFT")
	radiobutton.checkImage:SetWidth(height)
	radiobutton.checkImage:SetHeight(height)
	radiobutton.checkImage:SetTexture(UNCHECKED_TEXTURE)

	radiobutton.text = radiobutton:CreateFontString(nil,"OVERLAY","GameFontHighlightSmall")
	radiobutton.text:SetPoint("LEFT", radiobutton, "LEFT", height+1, 0)
	radiobutton.text:SetPoint("RIGHT", radiobutton, "RIGHT")
	radiobutton.text:SetJustifyH("LEFT")
	radiobutton.text:SetText(text)
	radiobutton.text:SetTextColor(1,1,1,1)
	Recount2:AddFontString(radiobutton.text)
	SetFontSize(radiobutton.text, height*0.8)

	radiobutton.value = false
	radiobutton.radioID = radioID
	radiobutton.radioValue = radioValue

	if radioID then
		if radioValues[radioID]==nil then
			radioValues[radioID] = radioValue
			radioFrames[radioID] = radiobutton
			radioFramesByValue[radioID] = {}
			radiobutton.checkImage:SetTexture(CHECKED_TEXTURE)
		end
		radioFramesByValue[radioID][radioValue] = radiobutton
	end

	radiobutton.OnEnter = OnEnter
	radiobutton.OnLeave = OnLeave
	radiobutton.OnMouseDown = OnMouseDown
	radiobutton.SetValue = SetValue

	radiobutton:SetScript("OnEnter", radiobutton.OnEnter)
	radiobutton:SetScript("OnLeave", radiobutton.OnLeave)
	radiobutton:SetScript("OnMouseDown", radiobutton.OnMouseDown)
	radiobutton:EnableMouse()

	return radiobutton
end

function Recount2_Widgets:GetRadioValue(radioID)
	return radioValues[radioID]
end

function Recount2_Widgets:SetRadioValue(radioID, radioValue)
	radioFramesByValue[radioID][radioValue]:SetValue(true)
end
