local Recount2_Widgets = Recount2.Widgets
local tinsert = table.insert

local function SetFontSize(string, size)
	local Font, Height, Flags = string:GetFont()
	string:SetFont(Font, size, Flags)
end

local function OnEnter(self)
	self.highlight:Show()
end

local function OnLeave(self)
	self.highlight:Hide()
end

local function OnMouseDown(self)
	local selected = self.parent.selected
	if selected~=self then
		selected.tabFrame:Hide()
		selected.background:SetTexture(1.0, 0.0, 0.0, 0.3)
		self.tabFrame:Show()
		self.background:SetTexture(0.0, 1.0, 0.0, 0.3)
		self.parent.selected = self

		self.parent.leftLine:SetPoint("BOTTOMRIGHT", self, "BOTTOMLEFT")
		self.parent.rightLine:SetPoint("BOTTOMLEFT", self, "BOTTOMRIGHT")
	end
end

local function AddTab(self, text)
	local tab = CreateFrame("FRAME", nil, self)
	tab.text = tab:CreateFontString(nil,"OVERLAY","GameFontHighlightSmall")
	tab.text:SetJustifyH("LEFT")
	tab.text:SetText(text)
	tab.text:SetTextColor(1,1,1,1)
	tab.text:SetPoint("CENTER", tab, "CENTER")
	Recount2:AddFontString(tab.text)
	SetFontSize(tab.text, self.height*0.8)

	tab.leftSide = tab:CreateTexture(nil, "BORDER")
	tab.leftSide:SetTexture("Interface\\Tooltips\\UI-Tooltip-Border.blp")
	tab.leftSide:SetTexCoord(0, 0.125, 0, 1)
	tab.leftSide:SetWidth(16)
	tab.leftSide:SetPoint("BOTTOMLEFT", tab, "BOTTOMLEFT", -3, 1)
	tab.leftSide:SetPoint("TOPLEFT", tab, "TOPLEFT", -3, -14)

	tab.rightSide = tab:CreateTexture(nil, "BORDER")
	tab.rightSide:SetTexture("Interface\\Tooltips\\UI-Tooltip-Border.blp")
	tab.rightSide:SetTexCoord(0.125, 0.25, 0, 1)
	tab.rightSide:SetWidth(16)
	tab.rightSide:SetPoint("BOTTOMRIGHT", tab, "BOTTOMRIGHT", 3, 1)
	tab.rightSide:SetPoint("TOPRIGHT", tab, "TOPRIGHT", 3, -14)

	tab.topSide = tab:CreateTexture(nil, "BORDER")
	tab.topSide:SetTexture("Interface\\Tooltips\\UI-Tooltip-Border.blp")
	tab.topSide:SetTexCoord(0.25, 1.0, 0.375, 1.0, 0.25, 0.0, 0.375, 0.0)
	tab.topSide:SetHeight(16)
	tab.topSide:SetPoint("TOPLEFT", tab, "TOPLEFT", 13, 2)
	tab.topSide:SetPoint("TOPRIGHT", tab, "TOPRIGHT", -13, 2)

	tab.topleft = tab:CreateTexture(nil, "BORDER")
	tab.topleft:SetTexture("Interface\\Tooltips\\UI-Tooltip-Border.blp")
	tab.topleft:SetTexCoord(0.5, 0.0, 0.5, 1.0, 0.625, 0.0, 0.625, 1.0)
	tab.topleft:SetHeight(16)
	tab.topleft:SetWidth(16)
	tab.topleft:SetPoint("TOPLEFT", tab, "TOPLEFT", -3, 2)

	tab.topright = tab:CreateTexture(nil, "BORDER")
	tab.topright:SetTexture("Interface\\Tooltips\\UI-Tooltip-Border.blp")
	tab.topright:SetTexCoord(0.625, 0.0, 0.625, 1.0, 0.75, 0.0, 0.75, 1.0)
	tab.topright:SetHeight(16)
	tab.topright:SetWidth(16)
	tab.topright:SetPoint("TOPRIGHT", tab, "TOPRIGHT", 3, 2)

	tab.background = tab:CreateTexture(nil, "BACKGROUND")
	tab.background:SetTexture(1.0, 0.0, 0.0, 0.3)
	tab.background:SetPoint("TOPLEFT", tab, "TOPLEFT", 1, -2)
	tab.background:SetPoint("BOTTOMRIGHT", tab, "BOTTOMRIGHT", -1, 1)

	tab.highlight = tab:CreateTexture(nil, "BACKGROUND")
	tab.highlight:SetTexture(1.0, 1.0, 0.0, 0.3)
	tab.highlight:SetBlendMode("ADD")
	tab.highlight:SetPoint("TOPLEFT", tab, "TOPLEFT", 1, -2)
	tab.highlight:SetPoint("BOTTOMRIGHT", tab, "BOTTOMRIGHT", -1, 1)
	tab.highlight:Hide()

	tab:SetWidth(tab.text:GetStringWidth()+4)
	tab:SetHeight(self.height)

	tab:SetScript("OnEnter", OnEnter)
	tab:SetScript("OnLeave", OnLeave)
	tab:SetScript("OnMouseDown", OnMouseDown)
	tab:EnableMouse()

	tab.tabFrame = CreateFrame("FRAME", nil, tab)

	tab.parent = self
	
	local numTabs = #self.tabs
	if numTabs>0 then
		tab:SetPoint("BOTTOMLEFT", self.tabs[numTabs], "BOTTOMRIGHT", 1, 0)
		tab.tabFrame:Hide()
	else
		tab:SetPoint("BOTTOMLEFT", self, "BOTTOMLEFT", 2, 0)
		self.leftLine:SetPoint("BOTTOMRIGHT", tab, "BOTTOMLEFT")
		self.rightLine:SetPoint("BOTTOMLEFT", tab, "BOTTOMRIGHT")
		tab.background:SetTexture(0.0, 1.0, 0.0, 0.3)
		self.selected = tab
	end

	tinsert(self.tabs, tab)

	return tab.tabFrame
end


function Recount2_Widgets:CreateTabs(parent, width, height)
	local tabs = CreateFrame("FRAME", nil, parent)

	tabs.height = height
	tabs:SetWidth(width)
	tabs:SetHeight(height)
	
	tabs.leftLine = tabs:CreateTexture(nil, "BACKGROUND")
	tabs.leftLine:SetTexture(0.7, 0.7, 0.7, 1.0)
	tabs.leftLine:SetHeight(1)
	tabs.leftLine:SetPoint("BOTTOMLEFT", tabs, "BOTTOMLEFT")
	tabs.leftLine:SetPoint("BOTTOMRIGHT", tabs, "BOTTOMLEFT")

	tabs.rightLine = tabs:CreateTexture(nil, "BACKGROUND")
	tabs.rightLine:SetTexture(0.7, 0.7, 0.7, 1.0)
	tabs.rightLine:SetHeight(1)
	tabs.rightLine:SetPoint("BOTTOMLEFT", tabs, "BOTTOMLEFT")
	tabs.rightLine:SetPoint("BOTTOMRIGHT", tabs, "BOTTOMRIGHT")

	tabs.tabs = {}
	tabs.AddTab = AddTab

	return tabs
end